// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
angular.module('starter', ['ionic', 'ionic.utils', 'starter.controllers', 'ngStorage', 'starter.services', 'ngQuickDate', 'pasvaz.bindonce', 'QuickList', 'pickadate'])
.run(['$ionicPlatform', '$rootScope', '$state', '$location', 'sync', '$localStorage', '$ls', '$document', function($ionicPlatform, $rootScope, $state, $location, sync, $localStorage, $ls, $document) {

  $rootScope.syncing = false;
  sync.setup();

	$rootScope.dev = false;
	$rootScope.forceBase = false;

	var user = $ls.getObject('user');

    var ua = navigator.userAgent.toLowerCase();
    var isAndroid = ua.indexOf("android") > -1;

// 	if ($rootScope.forceBase === true || (user && user.id))
  if ($rootScope.forceBase === true || (user && user.id === '2')) {
	if ($rootScope.dev === true) {
      	$rootScope.base = 'http://192.168.0.6';
	} else {
  	  $rootScope.base = isAndroid ? 'http://freshability.com' : 'https://freshability.com';
	}
  } else {
  	$rootScope.base = isAndroid ? 'http://freshability.com' : 'https://freshability.com';
  }

	$rootScope.uri = $rootScope.base + '/api';
	$rootScope.needToSync = $ls.get('needToSync');

	$document.on("resume", function( event ) {
		// do stuff with your $rootScope such as this:
		$rootScope.$broadcast('onResumeCordova');
	});

  $ionicPlatform.ready(function() {
    if(window.StatusBar) {
      // org.apache.cordova.statusbar required
      StatusBar.overlaysWebView(true);
      StatusBar.backgroundColorByHexString("#FFFFFF");
    }

    if (navigator.splashscreen) {
      setTimeout(function() {
          navigator.splashscreen.hide();
      }, 100);
    }

  });

  $rootScope.online = true;
	$rootScope.hasLocations = false;

	var locations = $ls.getObject('locations');

	if (locations && locations.length > 1) {
		$rootScope.hasLocations = true;
	}

}])

.config(function($stateProvider, $urlRouterProvider, $httpProvider, ngQuickDateDefaultsProvider, $ionicConfigProvider) {

	$ionicConfigProvider.backButton.text('').previousTitleText(false);

	ngQuickDateDefaultsProvider.set({
    parseDateFunction: function(str) {
      d = Date.create(str);
      return d.isValid() ? d : null;
    }
  });

  var interceptor = ['$localStorage', '$ls', function ($localStorage, $ls) {
    return {
      request: function (config){
	      var user = $ls.getObject('user');
        if (!config.headers.Authorization && user && user.auth_token) {
          config.headers.Authorization = 'Bearer ' + user.auth_token;
        } else if (!config.headers.Authorization) {
          config.headers.Authorization = 'Bearer none';
        }

    return config;
      }
    };
  }];

  $httpProvider.interceptors.push(interceptor);

  var auth = ['$rootScope', '$location', '$localStorage', '$ls', function($rootScope, $location, $localStorage, $ls) {
	  var user = $ls.getObject('user');
    if (user && user.id) {
      $rootScope.user = user;
      return true;
    }

      $location.url('/sign-in');
  }];

  var authLogin = ['$rootScope', '$location', '$localStorage', '$ls', function($rootScope, $location, $localStorage, $ls) {
	  var user = $ls.getObject('user');
    if (user && user.id) {
      $location.url('/audit');
    }
  }];

  $stateProvider

    .state('signin', {
      url: "/sign-in",
      templateUrl: "templates/sign-in.html",
      controller: 'SigninCtrl',
      resolve: authLogin
    })
    .state('logout', {
      url: "/logout",
      controller: ['$localStorage', '$ls', '$location', 'sync', function($localStorage, $ls, $location, sync) {
	      $ls.remove('user');
        $location.url('sign-in');
      }]
    })
		.state('pin', {
			url: '/pin',
			templateUrl: 'templates/pin',
			controller: 'PinCtrl'
		})
    .state('app', {
      url: "/app",
      abstract: true,
      templateUrl: "templates/menu.html",
      controller: 'AppCtrl'
    })
  .state('app.location', {
    url: '/location',
    views: {
      'menuContent' :{
        templateUrl: "templates/location.html",
        controller: 'LocationCtrl'
      }
    }
  })
    .state('app.audit', {
      url: "/audit",
      cache: false,
      views: {
        'menuContent' :{
          templateUrl: "templates/audit.html",
          controller: 'AuditCtrl'
        }
      },
      resolve: {
        authenticate: auth
      }
    })
  .state('app.newaudit', {
    url: '/newaudit',
    views: {
          'menuContent' :{
            templateUrl: "templates/newaudit.html",
            controller: 'NewAuditCtrl'
          }
    }
  })
  .state('app.editaudit', {
    url: '/editaudit/:id',
    views: {
          'menuContent' :{
            templateUrl: "templates/editaudit.html",
            controller: 'EditAuditCtrl'
          }
    }
  })
  .state('app.auditinfo', {
    url: '/auditinfo/:id',
    views: {
          'menuContent' :{
            templateUrl: "templates/auditinfo.html",
            controller: 'AuditInfoCtrl'
          }
    }
  })
  .state('app.auditimages', {
    url: '/auditimages/:id',
    views: {
          'menuContent' :{
            templateUrl: "templates/auditimages.html",
            controller: 'AuditImagesCtrl'
          }
    }
  })
  .state('app.auditsection', {
    url: '/auditsection/:tabId/:id',
    views: {
          'menuContent' :{
            templateUrl: "templates/auditsection.html",
            controller: 'AuditSectionCtrl'
          }
    }
  })
  .state('app.auditsection2', {
    url: '/auditsection2/:tabId/:id',
    views: {
          'menuContent' :{
            templateUrl: "templates/auditsection2.html",
            controller: 'AuditSection2Ctrl'
          }
    }
  })
	.state('app.data', {
		url: '/data',
		views: {
			'menuContent': {
				templateUrl: 'templates/data.html',
				controller: 'DataCtrl'
			}
		}
	})
    .state('app.settings', {
	    url: "/settings",
	    views: {
	          'menuContent' :{
	            templateUrl: "templates/settings.html",
	            controller: 'SettingsCtrl'
	          }
	    }
    });


  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/app/audit');
});

