angular.module('starter').directive('options', ['$timeout', 'u', 'sync', '$ls', function ($timeout, u, sync, $ls) {
	'use strict';

	return {
		restrict: 'E',
		templateUrl: 'templates/directive/option.html',
		scope: {
			item: '=',
			sectionInfo: '=',
			localAnswers: '=',
			key: '=',
			auditId: '='
		},
		controller: ['$rootScope', '$scope', '$http', '$ionicPopup', '$ionicModal', function ($rootScope, $scope, $http, $ionicPopup, $ionicModal) {
			$scope.newAnswer = {};
			$scope.a = {};

			$scope.localAnswerKey = $scope.auditId + ':::' + $scope.item.id;
			var localAnswer = $scope.localAnswers[$scope.localAnswerKey];

			if (localAnswer) {
				$scope.auditAnswer = localAnswer;
			} else if ($scope.sectionInfo.audit.answers[$scope.item.id]) {
				$scope.auditAnswer = $scope.sectionInfo.audit.answers[$scope.item.id];
			} else {
				$scope.auditAnswer = null;
			}

			if ($scope.auditAnswer) {
				$scope.newAnswer = angular.copy($scope.auditAnswer);
			} else {
					$scope.newAnswer = {
						guid: u.guid(),
						audit_question_id: $scope.item.id,
						edited: true,
						answer: null,
						audit_id: $scope.sectionInfo.audit.id
					};
			}

			var oldAnswer = $scope.newAnswer.answer;

			function _makeOpts() {
				$scope.opts = [];
				var matches;
				if (matches = $scope.item.opts.match(/([0-9]+)-([0-9]+)/)) {
					var start = parseInt(matches[1]);
					var end = parseInt(matches[2]);
					for (var i = start; i <= end; i++) {
						$scope.opts.push(String(i));
					}
				} else {
					angular.forEach($scope.item.opts.split(/,/), function(opt){
						opt = opt.trim();
						$scope.opts.push(opt);
					});
				}
			}

			function _setInitialAnswer() {
				if ( ! $scope.newAnswer.answer && $scope.item.defaultval) {
					$scope.newAnswer.answer = $scope.item.defaultval;
				}
			}

			if ($scope.item.type === 'dropdown') {
				_makeOpts();
				_setInitialAnswer();

			}

			if ($scope.item.type === 'multichoice') {
				_makeOpts();
				_setInitialAnswer();


				$scope.answers = [];
				if ($scope.newAnswer.answer !== null) {
					angular.forEach($scope.newAnswer.answer.split(/,/), function(answer){
						$scope.answers.push(answer.trim());
					});
				}
			}

			if ($scope.item.type === 'text' || $scope.item.type === 'text-na') {
				$scope.$watch('newAnswer.answer', function(val){
					if (val !== oldAnswer) {
						$scope.newAnswer.updated_at = u.phpts();
						$scope.sectionInfo.audit.answers[$scope.newAnswer.audit_question_id] = angular.copy($scope.newAnswer);
						sync.updateAudit($scope.sectionInfo.audit);
						oldAnswer = val;
					}
				});
			}

			$scope.showActions = false;

			$scope.isMultiAnswer = function(opt) {
				return $scope.answers.indexOf(opt) != -1;
			}

			$scope.setDropdownAnswer = function(opt) {
				$scope.newAnswer.answer = opt;
				$scope.newAnswer.updated_at = u.phpts();
				$scope.sectionInfo.audit.answers[$scope.newAnswer.audit_question_id] = angular.copy($scope.newAnswer);
				sync.updateAudit($scope.sectionInfo.audit);

				$scope.closeModal();
			}

			//Cleanup the modal when we're done with it!
			$scope.$on('$destroy', function() {
				 if ($scope.modal) {
					 $scope.modal.remove();
				 }
			});

			$scope.setMultiAnswer = function(opt) {
				var key = $scope.answers.indexOf(opt);
				if (key === -1) {
					$scope.answers.push(opt);
				} else {
					$scope.answers.splice(key, 1);
				}

				$scope.newAnswer.answer = $scope.answers.join(',');
				$scope.newAnswer.updated_at = u.phpts();

				$scope.sectionInfo.audit.answers[$scope.newAnswer.audit_question_id] = angular.copy($scope.newAnswer);
				sync.updateAudit($scope.sectionInfo.audit);
			}

			$scope.selectMulti = function() {
				var template = ($scope.item.type === 'multichoice') ? 'templates/directive/multiModal.html' : 'templates/directive/dropdownModal.html';

				$ionicModal.fromTemplateUrl(template, {
					scope: $scope,
					animation: 'slide-in-up'
				}).then(function(modal) {
					$scope.modal = modal;
					$scope.modal.show();
				});
			}

			$scope.closeModal = function() {
				$scope.modal.remove();
			}

			$scope.selectOpt = function(opt) {
				if ($scope.newAnswer.answer) {
					if (opt === $scope.newAnswer.answer) {
						return true;
					}
				} else if (opt === $scope.newAnswer.defaulval) {
					return true;
				}

				return false;
			}

			
		}],
		link: function($scope, $el, $attrs) {
			var parent = $el.parent().parent();
			var itemBlock = $el.find('div').eq(0);
			$scope.toggleAnswer = function(answer, other) {
				var newAnswerVal = $scope.newAnswer.answer === answer ? null : answer;

				if (newAnswerVal === '__na') {
					parent.addClass('sectionna');
					itemBlock.addClass('na');
				} else {
					parent.removeClass('sectionna');
					itemBlock.removeClass('na');
				}

				$scope.newAnswer.updated_at = u.phpts();
				$scope.newAnswer.answer = newAnswerVal;

				$scope.localAnswers[$scope.localAnswerKey] = angular.copy($scope.newAnswer);
				$ls.setObject('localAnswers', $scope.localAnswers);

				sync.updateAuditByKey($scope.sectionInfo.audit, $scope.key);
			}
		}
	}
}]);
