angular.module('starter').service('sync', ['$q', '$http', '$rootScope', 'u', '$ls', '$window', function ($q, $http, $rootScope, u, $ls, $window) {
	'use strict';

	var _this = this;
	_this.vars = ['audit_templates', 'audits', 'locations', 'localImages', 'localAnswers'];
	_this.varIsObj = ['localImages', 'localAnswers'];

	function setup() {
		if (!$ls.isset('locationId')) {
			$ls.set('locationId', null);
		}

		if (!$ls.isset('version')) {
			$ls.set('version', 1.0);
		}

		if (!$ls.isset('needToSync')) {
			$ls.set('locationId', false);
		}

		if (!$ls.isset('settings')) {
			$ls.setObject('settings', {
				requirePin: false
			});
		}

		for (var i = 0;i<_this.vars.length;i++) {
			var name = _this.vars[i];
			var isObj = _this.varIsObj.indexOf(name) !== -1;

			if (!$ls.isset(name)) {
				if (isObj) {
					$ls.setObject(name, {});
				} else {
					$ls.setArray(name, []);
				}
			}
		}

		var version = $ls.get('version');
		if (version !== 1.1) {
			//_this.upgrade();
		}
	}

	_this.upgrade = function() {
		$ls.set('version', 1.1);
	}

	function reset() {
		for (var i = 0; i < _this.vars.length; i++) {
			var name = _this.vars[i];

			if (_this.varIsObj) {
				$ls.setObject(name, {});
			} else {
				$ls.setArray(name, []);
			}
		}
	}

	function clearAll() {
		 $ls.$reset();
		 $ls.set('locationId', null);
	}

	function resetAll() {
		var deferred = $q.defer();
		for (var i = 0; i<_this.vars.length; i++) {
			var name = _this.vars[i];

			if (name === 'locations' || name === 'audit_templates') {
				 // Dont' reset locations or audit_templates
				 continue;
			}

			var isObj = _this.varIsObj.indexOf(name) !== -1;

			if (isObj) {
				$ls.setObject(name, {});
			} else {
				$ls.setArray(name, []);
			}

		}

		$ls.set('needToSync', false);
		$rootScope.needToSync = false;

		deferred.resolve();

		return deferred.promise;
	}

	function updateAudit(newAudit) {
		var audits = $ls.getArray('audits');

		angular.forEach(audits, function(audit, key){
			var guidMatch = false;
			if (newAudit.guid && newAudit.guid === audit.guid) {
				guidMatch = true;
			}

			var idMatch = false;
			if (newAudit.id && newAudit.id === audit.id) {
				idMatch = true;
			}

			if (idMatch || guidMatch) {
				audits[key] = angular.copy(newAudit);
				audits[key].updated = 1;
				audits[key].updated_at = u.phpts();
			}
		});

		$ls.setArray('audits', audits);
		$ls.set('needToSync', true);
		$rootScope.needToSync = true;
	}

	function updateAuditByKey(newAudit, key) {
		var record = angular.copy(newAudit);
		record.updated = 1;
		record.updated_at = u.phpts();

		var audits = $ls.getArray('audits');

		if (typeof answerKey !== 'undefined' && typeof answer !== 'undefined') {
			audits[key].answers[answerKey] = angular.copy(answer);
		} else {
			audits[key] = angular.copy(newAudit);
		}

		$ls.setArray('audits', audits);
		$ls.set('needToSync', true);
		$rootScope.needToSync = true;
	}

	function sync(type, id) {

		function _all() {
			 var localAnswers = $ls.getObject('localAnswers');
			 var audits = $ls.getArray('audits');

			var auditData = {
				multiple: true,
				data: audits,
				localAnswers: localAnswers
			};

			return $q.all([
				$http.post($rootScope.uri + '/audit', auditData).
					success(function(data){
						if (typeof data !== 'object') {
							return;
						}

						var win = function (r, key) {
							if (r.response) {
								var data = JSON.parse(r.response);
								u.saveRemoteImg(data, key);
							}
						}
						
						var fail = function (error) {
						}

						angular.forEach(data, function (remoteAudit) {
							var id;

							if (remoteAudit.guid) {
								// Prefer to find by guid
								id = remoteAudit.guid;
							} else {
								id = remoteAudit.id;
							}

							var key = u.findAuditKey(id);

							if (key === -1) {
								audits.push(remoteAudit);
							} else {
								audits[key] = angular.copy(remoteAudit);
							}
						});

						$ls.setArray('audits', audits);

						if ($window.navigator && $window.navigator.camera) {
							var localImages = $ls.getObject('localImages');
							var user = $ls.getObject('user');

							 angular.forEach(localImages, function(image, key){
								 var options = new FileUploadOptions();
								 options.fileKey = 'file';
								 options.mimeType = 'image/jpeg';
								 var params = image;
								 options.params = params;
	
								 var headers = {
									 Authorization: 'Bearer ' + user.auth_token
								 };
								 options.headers = headers;
	
								 var ft = new FileTransfer();
								 var url = $rootScope.uri + '/image';
								 ft.upload(image.local_uri, encodeURI(url), function(r){
								 	win(r, key);
								 }, fail, options);
							 });
						} else {
							$ls.setObject('localImages', {});
						}

						$ls.setObject('localAnswers', {});
						$ls.set('needToSync', false);

						$rootScope.needToSync = false;
						$rootScope.$broadcast('syncComplete');

					}),
				$http.get($rootScope.uri + '/location').
					success(function(data) {
						if (typeof data === 'object') {
							 $ls.setArray('locations', data);
						 }
					}),
				$http.get($rootScope.uri + '/audit_template').
					success(function(data) {
						if (typeof data === 'object') {
							 $ls.setArray('audit_templates', data);
						 }
					})
				]);
		}

		return _all();
	}

	_this.sync = sync;

	return {
		clearAll: clearAll,
		resetAll: resetAll,
		sync: sync,
		setup: setup,
		updateAudit: updateAudit,
		updateAuditByKey: updateAuditByKey
	};

}]);